@@ -16,10 +16,10 @@
 #include <xtl/xmultimethods.hpp>
 #include <xtensor/xarray.hpp>
 
-#include "zarray_impl.hpp"
 #include "zassign.hpp"
 #include "zfunction.hpp"
 #include "zmath.hpp"
+#include "zwrappers.hpp"
 
 namespace xt
 {
@@ -48,6 +48,17 @@ namespace xt
         zarray() = default;
         ~zarray() = default;
 
+        template <class T>
+        zarray(std::initializer_list<T> t);
+        template <class T>
+        zarray(std::initializer_list<std::initializer_list<T>> t);
+        template <class T>
+        zarray(std::initializer_list<
+                   std::initializer_list<
+                       std::initializer_list<T>
+                   >
+               > t);
+
         zarray(implementation_ptr&& impl);
         zarray& operator=(implementation_ptr&& impl);
 
@@ -84,6 +95,8 @@ namespace xt
 
         std::size_t dimension() const;
         const shape_type& shape() const;
+        void reshape(const shape_type& shape);
+        void reshape(shape_type&& shape);
         void resize(const shape_type& shape);
         void resize(shape_type&& shape);
         bool broadcast_shape(shape_type& shape, bool reuse_cache = false) const;
@@ -110,6 +123,9 @@ namespace xt
         implementation_ptr p_impl;
     };
 
+    zarray strided_view(zarray& z, xstrided_slice_vector& slices);
+    std::ostream& operator<<(std::ostream& out, const zarray& ar);
+
     /*************************
      * zarray implementation *
      *************************/
@@ -140,6 +156,28 @@ namespace xt
         return semantic_base::operator=(e);
     }
 
+    template <class T>
+    inline zarray::zarray(std::initializer_list<T> t)
+        : zarray(xarray<T>(t))
+    {
+    }
+
+    template <class T>
+    inline zarray::zarray(std::initializer_list<std::initializer_list<T>> t)
+        : zarray(xarray<T>(t))
+    {
+    }
+    
+    template <class T>
+    inline zarray::zarray(std::initializer_list<
+                              std::initializer_list<
+                                  std::initializer_list<T>
+                              >
+                          > t)
+        : zarray(xarray<T>(t))
+    {
+    }
+
     inline zarray::zarray(implementation_ptr&& impl)
         : p_impl(std::move(impl))
     {
@@ -161,7 +199,18 @@ namespace xt
         resize(rhs.shape());
         zassign_args args;
         args.trivial_broadcast = true;
-        zdispatcher_t<detail::xassign_dummy_functor, 1>::dispatch(*(rhs.p_impl), *p_impl, args);
+        if (p_impl->is_chunked())
+        {
+            auto l = [](zarray& lhs, const zarray& rhs, zassign_args& args)
+            {
+                zdispatcher_t<detail::xassign_dummy_functor, 1>::dispatch(*(rhs.p_impl), *(lhs.p_impl), args);
+            };
+            detail::run_chunked_assign_loop(*this, rhs, args, l);
+        }
+        else
+        {
+            zdispatcher_t<detail::xassign_dummy_functor, 1>::dispatch(*(rhs.p_impl), *p_impl, args);
+        }
         return *this;
     }
 
@@ -174,7 +223,18 @@ namespace xt
     {
         zassign_args args;
         args.trivial_broadcast = true;
-        zdispatcher_t<detail::xmove_dummy_functor, 1>::dispatch(*(rhs.p_impl), *p_impl, args);
+        if (p_impl->is_chunked())
+        {
+            auto l = [](zarray& lhs, const zarray& rhs, zassign_args& args)
+            {
+                zdispatcher_t<detail::xmove_dummy_functor, 1>::dispatch(*(rhs.p_impl), *(lhs.p_impl), args);
+            };
+            detail::run_chunked_assign_loop(*this, rhs, args, l);
+        }
+        else
+        {
+            zdispatcher_t<detail::xmove_dummy_functor, 1>::dispatch(*(rhs.p_impl), *p_impl, args);
+        }
         return *this;
     }
 
@@ -245,6 +305,16 @@ namespace xt
         return p_impl->shape();
     }
 
+    inline void zarray::reshape(const shape_type& shape)
+    {
+        p_impl->reshape(shape);
+    }
+
+    inline void zarray::reshape(shape_type&& shape)
+    {
+        p_impl->reshape(std::move(shape));
+    }
+
     inline void zarray::resize(const shape_type& shape)
     {
         p_impl->resize(shape);
@@ -280,6 +350,11 @@ namespace xt
         std::unique_ptr<zarray_impl> p(z.get_implementation().strided_view(slices));
         return zarray(std::move(p));
     }
+
+    inline std::ostream& operator<<(std::ostream& out, const zarray& ar)
+    {
+        return ar.get_implementation().print(out);
+    }
 }
 
 #endif
