@@ -11,7 +11,7 @@
 #define XTENSOR_ZASSIGN_HPP
 
 #include "xtensor/xassign.hpp"
-#include "zarray_impl.hpp"
+#include "zwrappers.hpp"
 
 namespace xt
 {
@@ -19,19 +19,38 @@ namespace xt
     {
         zassign_args();
         bool trivial_broadcast;
-        xstrided_slice_vector slices;
-        size_t chunk_index;
+        bool chunk_assign;
+        zchunked_iterator chunk_iter;
+
+        inline const xstrided_slice_vector& slices() const
+        {
+            return chunk_iter.get_slice_vector();
+        }
     };
 
     inline zassign_args::zassign_args()
         : trivial_broadcast(false)
-        , slices()
-        , chunk_index(0u)
+        , chunk_assign(false)
+        , chunk_iter()
     {
     }
 
     namespace detail
     {
+        template <class E1, class E2, class F>
+        void run_chunked_assign_loop(E1 & e1, const E2& e2, zassign_args& args, F f)
+        {
+            const zchunked_array& arr = e1.as_chunked_array();
+            args.chunk_iter = arr.chunk_begin();
+            args.chunk_assign = true;
+            auto chunk_end = arr.chunk_end();
+            while (args.chunk_iter != chunk_end)
+            {
+                f(e1, e2, args);
+                ++args.chunk_iter;
+            }
+        }
+
         template <class Tag>
         struct zexpression_assigner
         {
@@ -41,14 +60,11 @@ namespace xt
             {
                 if (e1.get_implementation().is_chunked())
                 {
-                    const zchunked_array& arr = e1.as_chunked_array();
-                    size_t grid_size = arr.grid_size();
-                    for (size_t i = 0; i < grid_size; ++i)
+                    auto l = [](E1& e1, const E2& e2, zassign_args& args)
                     {
-                        args.slices = arr.get_slice_vector(i);
-                        args.chunk_index = i;
                         e2.assign_to(e1.get_implementation(), args);
-                    }
+                    };
+                    run_chunked_assign_loop(e1, e2, args, l);
                 }
                 else
                 {
@@ -75,16 +91,16 @@ namespace xt
                 else if (impl.is_chunked())
                 {
                     using array_type = ztyped_chunked_array<value_type>;
-                    array_type& ar = static_cast<array_type&>(impl);
-                    size_t grid_size = ar.grid_size();
-                    for (size_t i = 0; i < grid_size; ++i)
+                    array_type& arr = static_cast<array_type&>(impl);
+                    auto chunk_end = arr.chunk_end();
+                    for (auto chunk_iter = arr.chunk_begin(); chunk_iter != chunk_end; ++chunk_iter)
                     {
-                        ar.assign_chunk(strided_view(e2, ar.get_slice_vector(i)), i);
+                        arr.assign_chunk(strided_view(e2, chunk_iter.get_slice_vector()), chunk_iter);
                     }
                 }
                 else
                 {
-                    using array_type = ztyped_array<value_type>;
+                    using array_type = ztyped_expression_wrapper<value_type>;
                     array_type& ar = static_cast<array_type&>(impl);
                     xarray<value_type> tmp(e2);
                     ar.assign(std::move(tmp));
@@ -129,16 +145,17 @@ namespace xt
         {
             zassign_wrapped_expression(lhs.get_array(), rhs, args);
         }
-        else if (!args.slices.empty())
+        else if (args.chunk_assign)
         {
             xarray<T> tmp(rhs);
             auto& chunked_lhs = static_cast<ztyped_chunked_array<T>&>(lhs);
-            chunked_lhs.assign_chunk(std::move(tmp), args.chunk_index); 
+            chunked_lhs.assign_chunk(std::move(tmp), args.chunk_iter); 
         }
         else
         {
             xarray<T> tmp(rhs);
-            lhs.assign(std::move(tmp));
+            auto& expr_lhs = static_cast<ztyped_expression_wrapper<T>&>(lhs);
+            expr_lhs.assign(std::move(tmp));
         }
     }
 }
